<?php

namespace Drupal\layout_builder_block_delete;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides batch processing for deleting blocks from nodes.
 */
class LayoutBuilderBlockDeleteBatch implements ContainerInjectionInterface {

  use StringTranslationTrait;

  /**
   * The layout builder block delete manager.
   *
   * @var \Drupal\layout_builder_block_delete\LayoutBuilderBlockDeleteManager
   */
  protected $layoutBuilderBlockDeleteManager;

  /**
   * Constructs a new SanitizerForm object.
   */
  public function __construct(LayoutBuilderBlockDeleteManager $layout_builder_block_delete_manager) {
    $this->layoutBuilderBlockDeleteManager = $layout_builder_block_delete_manager;
  }

  /**
   * Create method.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('layout_builder_block_delete.manager')
    );
  }

  /**
   * Kick off batch process to delete blocks on all nodes for content types.
   */
  public function batchStart(array $content_types) {
    $nodes = [];

    foreach ($content_types as $content_type) {
      if ($content_type) {
        $nodes += $this->layoutBuilderBlockDeleteManager->getNodesForContentType($content_type);
      }
    }

    if (count($nodes)) {
      batch_set([
        'title' => $this->t('Deleting blocks'),
        'init_message' => $this->t('Starting blocks deletion'),
        'operations' => [
          [
            [
              'Drupal\layout_builder_block_delete\LayoutBuilderBlockDeleteBatch',
              'batchDeleteAllBlocks',
            ],
            [
              array_keys($nodes),
            ],
          ],
        ],
        'finished' => [
          'Drupal\layout_builder_block_delete\LayoutBuilderBlockDeleteBatch',
          'batchCompleted',
        ],
      ]);
    }
  }

  /**
   * Load nodes in batch process progressively to delete all blocks.
   */
  public static function batchDeleteAllBlocks($nids, &$context) {
    if (empty($context['sandbox'])) {
      // Flush caches to avoid false positives looking for block UUID.
      drupal_flush_all_caches();
      $context['sandbox'] = [];
      $context['sandbox']['progress'] = 0;
      $context['sandbox']['current_node'] = 0;
      // Save node count for the termination message.
      $context['sandbox']['max'] = count($nids);
    }

    $limit = 5;

    // Retrieve the next group.
    $current_node = $context['sandbox']['current_node'];
    $limit_node = $context['sandbox']['current_node'] + $limit;
    $result = range($current_node, $limit_node);

    foreach ($result as $row) {
      /** @var \Drupal\layout_builder_block_delete\LayoutBuilderBlockDeleteManager $layout_builder_block_delete_manager */
      $layout_builder_block_delete_manager = \Drupal::service('layout_builder_block_delete.manager');

      if (!empty($nids[$row])) {
        $layout_builder_block_delete_manager->deleteBlocks($nids[$row]);
      }

      $operation_details = t('Deleting blocks for nodes :current to :limit', [
        ':current' => $current_node,
        ':limit' => $limit_node,
      ]);

      // Store some results for post-processing in the 'finished' callback.
      // The contents of 'results' will be available as $results in the
      // 'finished' function.
      $context['results'][] = $row;

      // Update our progress information.
      $context['sandbox']['progress']++;
      $context['sandbox']['current_node'] = $row;
      $context['message'] = t('@details', [
        '@details' => $operation_details,
      ]);
    }

    // Inform the batch engine that we are not finished,
    // and provide an estimation of the completion level we reached.
    if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
      $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
    }
  }

  /**
   * Callback for batch processing completed.
   */
  public static function batchCompleted($success, $results, $operations) {
    $messenger = \Drupal::messenger();
    if ($success == TRUE) {
      $messenger->addMessage(t('@count nodes were processed.', [
        '@count' => count($results),
      ]));
    }
    else {
      // An error occurred.
      $messenger->addMessage(t('An error occurred while processing.'));
    }
  }

}